@@ -252,7 +252,8 @@ class DataFrame:
     def __to_np_arrays__(self,has_header = True):
         shape = self.shape()
         columns = np.empty(shape[1],list)
-        lines = self.values(clear = False).split("\n")
+        lines = self.values(clear = False)
+        lines = lines.split("\n")
         str_types = self.get_types().values()
         types = []
         types_text = str_types.split(",")
@@ -262,7 +263,10 @@ class DataFrame:
         names = self.columns
         for line in lines:
             if line != "":
-                elements = line.split(self.delimiter)
+                command = "echo '" +line.replace("\n","") + "' | gawk -v FPAT='([^" + self.delimiter + "]*)|(\"[^\"]+\")' 'BEGIN{}{for (i=1;i<=NF;i++){print($i)}}END{}' "
+                result = os.popen(command).read()
+                elements = result.split("\n")
+                elements = elements[0:len(elements)-1]
                 if line_counter == 0:
                         if has_header:
                             names = elements
@@ -313,6 +317,10 @@ class DataFrame:
         record_delimiter = "\\n"
         record_delimiter_transform = "\\n"
         intermediate_record_delimiter = "\\n"
+        if len(self.commands) == 0:
+            self.get_rows(range(self.__nrow_original__),return_other_object=False)
+            self.commands[0].persistance_in_time = "instance"
+            self.commands[0].persistance_after_execution = "ephemeral"
         for command in self.commands:
             if output_header == 0:
                 has_header = 0
@@ -352,7 +360,7 @@ class DataFrame:
             self.clear_commands()
         return result
 
-    def __get_rows_from_to__(self,min_row,max_row,has_header = True,output_header = True):
+    def __get_rows_from_to__(self,min_row,max_row,has_header = True,output_header = True,return_other_object = True):
 
         awk_command = Awk_command()
         
@@ -427,19 +435,20 @@ class DataFrame:
             awk_command.priority = 1
             awk_command.type = "get_rows_range"
             self.commands.append(awk_command)
-            __ddf__ = deepcopy(self)
-            __ddf__.__settle_commands__()
-            self.clear_commands()
-            return __ddf__
+            if return_other_object:
+                __ddf__ = deepcopy(self)
+                __ddf__.__settle_commands__()
+                self.clear_commands()
+                return __ddf__
         else:
             raise Exception("No rows selected")
 
-    def get_rows(self,rows,has_header = True,output_header = True):
+    def get_rows(self,rows,has_header = True,output_header = True,return_other_object = True):
         
         
         
         if type(rows) == range:
-            return DataFrame.__get_rows_from_to__(self,min(rows),max(rows),has_header = has_header,output_header = output_header)
+            return DataFrame.__get_rows_from_to__(self,min(rows),max(rows),has_header = has_header,output_header = output_header,return_other_object = return_other_object)
         else:
             
             awk_command = Awk_command()
@@ -526,10 +535,11 @@ class DataFrame:
             awk_command.priority = 1
             awk_command.type = "get_rows"
             self.commands.append(awk_command)
-            __ddf__ = deepcopy(self)
-            __ddf__.__settle_commands__()
-            self.clear_commands()
-            return __ddf__
+            if return_other_object:
+                __ddf__ = deepcopy(self)
+                __ddf__.__settle_commands__()
+                self.clear_commands()
+                return __ddf__
 
     def get_cols(self,cols,output_has_header = True,output_header = True):
         awk_command = Awk_command()
@@ -670,7 +680,8 @@ class DataFrame:
         for command in self.commands:
             command.persistance_after_execution = "permanent"
 
-    def get_types(self,num_rows_to_check = 1000):
+    def get_types(self):
+        print("TODO: missing checking for header")
         awk_command = Awk_command()
         command = "gawk -v FPAT='([^" + self.delimiter + "]*)|(\"[^\"]+\")' " + """'
         BEGIN {
@@ -686,19 +697,21 @@ class DataFrame:
             }else{
 
                 for (i=1;i<=NF;i++){
-                    if ($i ~ /^[0-9]+$/){
-                        types[$i] = "int"
+                    if ($i == ""){
+                        types[$i] = "str"
                     }else{
-                        if ($i ~ /^[0-9]*[.][0-9]+$/){
-                            types[$i] = "float"
+                        if ($i ~ /^[0-9]+$/ && types[$i] == "int"){
+                            types[$i] = "int"
                         }else{
-                            types[$i] = "str"
+                            if (($i ~ /^[0-9]*[.][0-9]+$/ || $i=="Nan" || $i=="nan" || $i=="NaN" || $i=="NAN") && (types[$i] == "int" || types[$i] == "float")){
+                                types[$i] = "float"
+                            }else{
+                                types[$i] = "str"
+                            }
                         }
                     }
                 }
-                if (NR > """ + str(num_rows_to_check) + """){
-                    exit
-                }
+               
             }
         }
         END {
